package com.jpsoft.printing.modules;

import com.jpsoft.printing.config.OSSConfig;
import com.jpsoft.printing.modules.base.dto.YearSoaDTO;
import com.jpsoft.printing.modules.base.entity.Account;
import com.jpsoft.printing.modules.base.entity.Customer;
import com.jpsoft.printing.modules.base.entity.Stock;
import com.jpsoft.printing.modules.base.entity.Work;
import com.jpsoft.printing.modules.base.service.AccountService;
import com.jpsoft.printing.modules.base.service.CustomerService;
import com.jpsoft.printing.modules.base.service.StockService;
import com.jpsoft.printing.modules.base.service.WorkService;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/report")
@Api(description = "report")
public class ReportController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OSSConfig ossConfig;
    @Autowired
    private WorkService workService;
    @Autowired
    private StockService stockService;
    @Autowired
    private AccountService accountService;
    @Autowired
    private CustomerService customerService;

    @ApiOperation(value="交易榜单")
    @RequestMapping(value = "transactionRanking",method = RequestMethod.POST)
    public MessageResult transactionRanking(String startDay, String endDay, Integer limit){
        MessageResult msgResult = new MessageResult<>();

        List<Account> list = accountService.findTransactionList(null,0, startDay, endDay, "total_amount ASC", limit);
        for(Account account : list){
            account.setTotalAmount(account.getTotalAmount().negate());

            Customer customer = customerService.get(account.getCustomerId());
            if(customer != null) {
                account.setCustomerName(customer.getCompany());
            }
            else {
                account.setCustomerName(null);
            }
        }

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }

    @ApiOperation(value="导出交易榜单")
    @RequestMapping(value = "transactionRankingXls",method = RequestMethod.POST)
    public MessageResult transactionRankingXls(String startDay, String endDay, Integer limit, String title) throws Exception {
        MessageResult msgResult = new MessageResult<>();

        List<Account> list = accountService.findTransactionList(null, 0, startDay, endDay, "total_amount ASC", limit);
        for(Account account : list){
            account.setTotalAmount(account.getTotalAmount().negate());

            Customer customer = customerService.get(account.getCustomerId());
            if(customer != null) {
                account.setCustomerName(customer.getCompany());
            }
            else {
                account.setCustomerName(null);
            }
        }

        String xlsPath = "static/transactionRanking.xls";
        ClassPathResource resource = new ClassPathResource(xlsPath);
        Workbook wb = WorkbookFactory.create(resource.getInputStream());

        //写入数据
        Sheet sheet = wb.getSheetAt(0);
        Row rowTitle = sheet.getRow(0);
        rowTitle.getCell(0).setCellValue(title);
        int rowNum = 2;

        for(int i=0; i<list.size(); i++) {
            Account account = list.get(i);

            Row row = sheet.createRow(rowNum);
            row.createCell(0).setCellValue(i+1);
            row.createCell(1).setCellValue(account.getCustomerName());
            row.createCell(2).setCellValue(account.getTotalAmount().toString());

            rowNum++;
        }

        //保存到服务器，返回文件名
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        wb.write(output);
        String fileName = title + "（" + sdf.format(new Date()) + "）.xls";
        String name = "D:\\JpSoft\\Tomcat 8.5\\webapps\\printing-portal\\xls\\" + fileName;
        File file = new File(name);
        FileOutputStream file1 = new FileOutputStream(file);
        wb.write(file1);
        file1.close();
        wb.close();
        output.close();

        msgResult.setResult(true);
        msgResult.setMessage("报表生成完成。");
        msgResult.setData("/printing-portal/xls/" + fileName);

        return msgResult;
    }

    @ApiOperation(value="交易报表-老板")
    @RequestMapping(value = "transactionReportBoss",method = RequestMethod.POST)
    public MessageResult transactionReportBoss(String startDay, String endDay){
        MessageResult msgResult = new MessageResult<>();

        List<Account> list = accountService.findTransactionList(null, 0, startDay, endDay, "b.name_ ASC", null);
        for(Account account : list){
            account.setTotalAmount(account.getTotalAmount().negate());

            Customer customer = customerService.get(account.getCustomerId());
            if(customer != null) {
                account.setCustomerName(customer.getCompany());
            }
            else {
                account.setCustomerName(null);
            }
        }

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }

    @ApiOperation(value="导出交易报表-老板")
    @RequestMapping(value = "transactionReportBossXls",method = RequestMethod.POST)
    public MessageResult transactionReportBossXls(String startDay, String endDay, String title) throws Exception {
        MessageResult msgResult = new MessageResult<>();

        List<Account> list = accountService.findTransactionList(null, 0, startDay, endDay, "b.name_ ASC", null);
        for(Account account : list){
            account.setTotalAmount(account.getTotalAmount().negate());

            Customer customer = customerService.get(account.getCustomerId());
            if(customer != null) {
                account.setCustomerName(customer.getCompany());
            }
            else {
                account.setCustomerName(null);
            }
        }

        String xlsPath = "static/transactionReportBoss.xls";
        ClassPathResource resource = new ClassPathResource(xlsPath);
        Workbook wb = WorkbookFactory.create(resource.getInputStream());

        //写入数据
        Sheet sheet = wb.getSheetAt(0);
        Row rowTitle = sheet.getRow(0);
        rowTitle.getCell(0).setCellValue(title);
        int rowNum = 2;

        for(int i=0; i<list.size(); i++) {
            Account account = list.get(i);

            Row row = sheet.createRow(rowNum);
            row.createCell(0).setCellValue(i+1);
            row.createCell(1).setCellValue(account.getCustomerName());
            row.createCell(2).setCellValue(account.getActualAmount().toString());
            row.createCell(3).setCellValue(account.getClothAmount().toString());
            row.createCell(4).setCellValue(account.getRollAmount().toString());
            row.createCell(5).setCellValue(account.getTotalAmount().toString());

            rowNum++;
        }

        //保存到服务器，返回文件名
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        wb.write(output);
        String fileName = title + "（" + sdf.format(new Date()) + "）.xls";
        String name = "D:\\JpSoft\\Tomcat 8.5\\webapps\\printing-portal\\xls\\" + fileName;
        File file = new File(name);
        FileOutputStream file1 = new FileOutputStream(file);
        wb.write(file1);
        file1.close();
        wb.close();
        output.close();

        msgResult.setResult(true);
        msgResult.setMessage("报表生成完成。");
        msgResult.setData("/printing-portal/xls/" + fileName);

        return msgResult;
    }

    @ApiOperation(value="年度对账单")
    @RequestMapping(value = "yearSOA",method = RequestMethod.POST)
    public MessageResult yearSOA(String customerId, Date selectDate){
        MessageResult msgResult = new MessageResult<>();
        Map<String, Object> map = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdfEx = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        Customer customer = customerService.get(customerId);
        if (StringUtils.isNotEmpty(customer.getCompany())) {
            map.put("name", customer.getName() + "（" + customer.getCompany() + "）");
        } else {
            map.put("name", customer.getName());
        }
        map.put("year", sdf.format(selectDate).substring(0,4));

        List<YearSoaDTO> list = new ArrayList<>();

        BigDecimal debitAmount = new BigDecimal(0);
        BigDecimal creditAmount = new BigDecimal(0);
        for(int i=0; i<12; i++) {
            Calendar calendarStart = Calendar.getInstance();
            calendarStart.setTime(selectDate);
            calendarStart.add(Calendar.MONTH, i);
            String dateStart = sdf.format(calendarStart.getTime());

            Calendar calendarEnd = Calendar.getInstance();
            calendarEnd.setTime(selectDate);
            calendarEnd.add(Calendar.MONTH, i+1);
            String dateEnd = sdf.format(calendarEnd.getTime());

            if(i == 0) {
                YearSoaDTO dtoStart = new YearSoaDTO();
                dtoStart.setSummary("承上页");
                dtoStart.setBalance(accountService.getBalance(customerId, dateStart));
                list.add(dtoStart);
            }

            List<Account> accountList = accountService.findList(customerId, null, dateStart, dateEnd, "create_time ASC");
            if(accountList.size() > 0) {
                String month = "";
                String day = "";
                String planNumber = "";
                BigDecimal actualAmount = new BigDecimal(0);
                BigDecimal clothAmount = new BigDecimal(0);
                BigDecimal rollAmount = new BigDecimal(0);
                BigDecimal totalAmount = new BigDecimal(0);
                Integer temp = -1;
                for (int j = 0; j < accountList.size(); j++) {
                    Account account = accountList.get(j);
                    if (account.getType() != temp) {
                        month = sdf.format(account.getCreateTime()).substring(5, 7);
                        day = sdf.format(account.getCreateTime()).substring(8, 10);
                        planNumber += account.getPlanNumber() + " ";
                        actualAmount = actualAmount.add(account.getActualAmount());
                        clothAmount = clothAmount.add(account.getClothAmount());
                        rollAmount = rollAmount.add(account.getRollAmount());
                        totalAmount = totalAmount.add(account.getTotalAmount());

                        temp = account.getType();
                    } else {
                        planNumber += account.getPlanNumber() + " ";
                        actualAmount = actualAmount.add(account.getActualAmount());
                        clothAmount = clothAmount.add(account.getClothAmount());
                        rollAmount = rollAmount.add(account.getRollAmount());
                        totalAmount = totalAmount.add(account.getTotalAmount());
                    }

                    if (j + 1 == accountList.size() || account.getType() != accountList.get(j + 1).getType()) {
                        Calendar calendarNow = Calendar.getInstance();
                        calendarNow.setTime(account.getCreateTime());
                        calendarNow.add(Calendar.SECOND, 1);

                        YearSoaDTO dto = new YearSoaDTO();
                        dto.setMonth(month);
                        dto.setDay(day);
                        switch (account.getType()) {
                            case 0:
                                debitAmount = debitAmount.add(totalAmount);
                                dto.setPlanNumber(planNumber);
                                dto.setSummary(String.format("加工金额：%s      坯布金额：%s      力工金额：%s", actualAmount, clothAmount, rollAmount));
                                dto.setDebit(totalAmount);
                                break;
                            case 1:
                                creditAmount = creditAmount.add(totalAmount);
                                dto.setPlanNumber("收款");
                                dto.setSummary("收款");
                                dto.setCredit(totalAmount);
                                break;
                            case 2:
                                debitAmount = debitAmount.add(totalAmount);
                                dto.setPlanNumber("冲账");
                                dto.setSummary("平账");
                                dto.setDebit(totalAmount);
                                break;
                        }
                        String str = sdfEx.format(calendarNow.getTime());

                        dto.setBalance(accountService.getBalance(customerId, sdfEx.format(calendarNow.getTime())));
                        list.add(dto);

                        month = "";
                        day = "";
                        planNumber = "";
                        actualAmount = new BigDecimal(0);
                        clothAmount = new BigDecimal(0);
                        rollAmount = new BigDecimal(0);
                        totalAmount = new BigDecimal(0);
                    }
                }
            }

            if(i == 11) {
                YearSoaDTO dtoEnd = new YearSoaDTO();
                dtoEnd.setMonth("合计");
                dtoEnd.setSummary("启下页");
                dtoEnd.setDebit(debitAmount);
                dtoEnd.setCredit(creditAmount);
                dtoEnd.setBalance(accountService.getBalance(customerId, dateEnd));
                list.add(dtoEnd);
            }
        }

        map.put("list", list);
        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }

    @ApiOperation(value="年度对账单报表")
    @RequestMapping(value = "yearSOAXls",method = RequestMethod.POST)
    public MessageResult yearSOAXls(String customerId, Date selectDate) throws Exception {
        MessageResult msgResult = new MessageResult<>();

        Map<String, Object> map = (Map<String, Object>)yearSOA(customerId, selectDate).getData();

        String xlsPath = "static/wlmxz.xls";
        ClassPathResource resource = new ClassPathResource(xlsPath);
        Workbook wb = WorkbookFactory.create(resource.getInputStream());

        //写入数据
        Sheet sheet = wb.getSheetAt(0);
        Row rowTitle1 = sheet.getRow(0);
        rowTitle1.getCell(0).setCellValue("往来明细账--" + map.get("name"));
        Row rowTitle2 = sheet.getRow(1);
        rowTitle2.getCell(0).setCellValue(map.get("year") + "年");
        int rowNum = 3;

        BigDecimal debit = new BigDecimal(0);
        BigDecimal credit = new BigDecimal(0);
        BigDecimal balance = new BigDecimal(0);
        List<YearSoaDTO> list = (List<YearSoaDTO>)map.get("list");
        for(int i=0; i<list.size(); i++) {
            YearSoaDTO yearSoaDTO = list.get(i);

            Row row = sheet.createRow(rowNum);
            row.createCell(0).setCellValue(yearSoaDTO.getMonth());
            row.createCell(1).setCellValue(yearSoaDTO.getDay());
            row.createCell(2).setCellValue(yearSoaDTO.getPlanNumber());
            row.createCell(3).setCellValue(yearSoaDTO.getSummary());
            if(yearSoaDTO.getDebit() != null) {
                debit = debit.add(yearSoaDTO.getDebit());
                row.createCell(4).setCellValue(yearSoaDTO.getDebit().doubleValue());
            }
            if(yearSoaDTO.getCredit() != null) {
                credit = credit.add(yearSoaDTO.getCredit());
                row.createCell(6).setCellValue(yearSoaDTO.getCredit().doubleValue());
            }
            if(yearSoaDTO.getBalance() != null) {
                balance = balance.add(yearSoaDTO.getBalance());
                row.createCell(8).setCellValue(yearSoaDTO.getBalance().doubleValue());
            }

            rowNum++;
        }
//        Row rowTitle1 = sheet.createRow(rowNum);
//        rowTitle1.createCell(0).setCellValue("合计");

        //保存到服务器，返回文件名
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        wb.write(output);
        String fileName = "往来明细账（" + sdf.format(new Date()) + "）.xls";
        String name = "D:\\JpSoft\\Tomcat 8.5\\webapps\\printing-portal\\xls\\" + fileName;
        File file = new File(name);
        FileOutputStream file1 = new FileOutputStream(file);
        wb.write(file1);
        file1.close();
        wb.close();
        output.close();

        msgResult.setResult(true);
        msgResult.setMessage("报表生成完成。");
        msgResult.setData("/printing-portal/xls/" + fileName);

        return msgResult;
    }

    @ApiOperation(value="发货结算表数据")
    private Map dataFHJSB(String customerId, Date selectDate){
        Map<String, Object> map = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(selectDate);
        String dateStart = sdf.format(calendar.getTime());

        calendar.add(Calendar.MONTH, 1);
        String dateEnd = sdf.format(calendar.getTime());

        Customer customer = customerService.get(customerId);

        List<Map> mapList = new ArrayList<>();
        List<Work> workList = workService.findList(dateStart, dateEnd, "create_time ASC");
        for(Work work : workList){
            Map<String, Object> accMap = new HashMap<>();

            //账目
            Account account = accountService.getByWorkId(work.getId());
            if(account != null){
                accMap.put("actualAmount", account.getActualAmount());//实数金额
                accMap.put("clothAmount", account.getClothAmount());//坯布金额
                accMap.put("rollAmount", account.getRollAmount());//力工金额
                accMap.put("totalAmount", account.getTotalAmount());//总金额
            }
            else{
                accMap.put("actualAmount", "未出账单");
                accMap.put("clothAmount", "未出账单");
                accMap.put("rollAmount", "未出账单");
                accMap.put("totalAmount", "未出账单");
            }

            //打卷
            BigDecimal actualQuantityM = new BigDecimal(0);
            List<String> lengthList = new ArrayList<>();
            List<String> shipmentList = new ArrayList<>();
            List<Stock> stockList = stockService.findList(work.getId());
            for(Stock stock : stockList){
                lengthList.add(stock.getTagLength() + "m");
                actualQuantityM = actualQuantityM.add(stock.getLength());
                if(stock.getUpdateTime() != null && !shipmentList.contains(sdf.format(stock.getUpdateTime()))) {
                    shipmentList.add(sdf.format(stock.getUpdateTime()));
                }
            }
            BigDecimal actualQuantityY = actualQuantityM.divide(new BigDecimal("0.9144"), 0, RoundingMode.HALF_UP);

            accMap.put("name", work.getName());//品种及规格
            accMap.put("width", work.getWidth());//幅宽
            accMap.put("number", work.getNumber());//编号
            accMap.put("processDate", sdf.format(work.getProcessDate()));//投坯日
            accMap.put("processVolume", work.getProcessVolume());//投坯数（米）
            accMap.put("processRemark", work.getProcessRemark());//投坯备注
            accMap.put("processNumber", work.getProcessNumber());//投坯卷数
            accMap.put("ratio", work.getRatio());//伸长率
            accMap.put("colour", work.getColour());//色泽
            accMap.put("estimateQuantity", work.getEstimateQuantity());//应交成品数（米）
            accMap.put("estimateRemark", work.getEstimateRemark());//应交成品数备注
            accMap.put("shipment", String.join(",", shipmentList));//发货日
            accMap.put("actualRemark", String.join("+", lengthList));//实交成品数-详情
            accMap.put("actualQuantityY", actualQuantityY);//实交成品数（码）
            accMap.put("actualQuantityM", actualQuantityM);//实交成品数（米）
            accMap.put("unitPrice", work.getUnitPrice());//单价（元/米）
            accMap.put("clothQuantity", actualQuantityM.subtract(work.getEstimateQuantity()));//坯布数（米）
            accMap.put("clothPrice", work.getClothPrice());//坯布单价（元/米）
            accMap.put("remark", work.getRemark());//备注
            mapList.add(accMap);
        }

        map.put("name", "客户：" + customer.getCompany());
        map.put("date", dateStart.substring(0,4) + "年" + dateStart.substring(5,7) + "月");
        map.put("list", mapList);

        return map;
    }

    @ApiOperation(value="发货结算表")
    @RequestMapping(value = "monthFHJSB",method = RequestMethod.POST)
    public MessageResult monthFHJSB(String customerId, Date selectDate){
        MessageResult msgResult = new MessageResult<>();

        Map map = dataFHJSB(customerId, selectDate);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }

    @ApiOperation(value="发货结算表报表")
    @RequestMapping(value = "monthFHJSBXls",method = RequestMethod.POST)
    public MessageResult monthFHJSBXls(String customerId, Date selectDate) throws Exception {
        MessageResult msgResult = new MessageResult<>();

        Map<String, Object> map = (Map<String, Object>)dataFHJSB(customerId, selectDate);

        String xlsPath = "static/fhjsb.xls";
        ClassPathResource resource = new ClassPathResource(xlsPath);
        Workbook wb = WorkbookFactory.create(resource.getInputStream());

        //写入数据
        Sheet sheet = wb.getSheetAt(0);
        Row rowTitle = sheet.getRow(1);
        rowTitle.getCell(0).setCellValue(map.get("name").toString());
        rowTitle.getCell(20).setCellValue(map.get("date").toString());
        int rowNum = 4;

        List<Map> list = (List<Map>)map.get("list");
        for(Map item : list) {
            Row row = sheet.createRow(rowNum);
            row.createCell(0).setCellValue(item.get("name").toString());
            row.createCell(1).setCellValue(item.get("width").toString());
            row.createCell(2).setCellValue(item.get("number").toString());
            row.createCell(3).setCellValue(item.get("processDate").toString());
            row.createCell(4).setCellValue(item.get("processVolume").toString());
            row.createCell(5).setCellValue(item.get("processNumber").toString());
            row.createCell(6).setCellValue(item.get("processRemark").toString());
            row.createCell(7).setCellValue(item.get("ratio").toString());
            row.createCell(8).setCellValue(item.get("colour").toString());
            row.createCell(9).setCellValue(item.get("estimateQuantity").toString());
            row.createCell(10).setCellValue(item.get("shipment").toString());
            row.createCell(11).setCellValue(item.get("actualRemark").toString());
            row.createCell(12).setCellValue(item.get("actualQuantityY").toString());
            row.createCell(13).setCellValue(item.get("actualQuantityM").toString());
            row.createCell(14).setCellValue(item.get("unitPrice").toString());
            row.createCell(15).setCellValue(item.get("actualAmount").toString());
            row.createCell(16).setCellValue(item.get("clothQuantity").toString());
            row.createCell(17).setCellValue(item.get("clothPrice").toString());
            row.createCell(18).setCellValue(item.get("clothAmount").toString());
            row.createCell(19).setCellValue(item.get("rollAmount").toString());
            row.createCell(20).setCellValue(item.get("totalAmount").toString());
            row.createCell(21).setCellValue(item.get("remark").toString());

            rowNum++;
        }

        //保存到服务器，返回文件名
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        wb.write(output);
        String fileName = "发货结算表（" + sdf.format(new Date()) + "）.xls";
        String name = "D:\\JpSoft\\Tomcat 8.5\\webapps\\printing-portal\\xls\\" + fileName;
        File file = new File(name);
        FileOutputStream file1 = new FileOutputStream(file);
        wb.write(file1);
        file1.close();
        wb.close();
        output.close();

        msgResult.setResult(true);
        msgResult.setMessage("报表生成完成。");
        msgResult.setData("/printing-portal/xls/" + fileName);

        return msgResult;
    }
}
