package com.jpsoft.printing.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.base.entity.Customer;
import com.jpsoft.printing.modules.base.service.CustomerService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/customer")
@Api(description = "customer")
public class CustomerController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private SysLogService sysLogService;
    @Autowired
    private CustomerService customerService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Customer> create(){
        MessageResult<Customer> msgResult = new MessageResult<>();

        Customer customer = new Customer();

        msgResult.setData(customer);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Customer> add(@RequestBody Customer customer,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Customer> msgResult = new MessageResult<>();

        try {
            customer.setId(UUID.randomUUID().toString());
            customer.setDelFlag(false);
            customer.setCreateBy(subject);
            customer.setCreateTime(new Date());
            
            int affectCount = customerService.insert(customer);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(customer);
                msgResult.setMessage("数据库添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),customer.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Customer> edit(@PathVariable("id") String id){
        MessageResult<Customer> msgResult = new MessageResult<>();

        try {
            Customer customer = customerService.get(id);

            if (customer != null) {
                msgResult.setResult(true);
                msgResult.setData(customer);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Customer> update(@RequestBody Customer customer,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Customer> msgResult = new MessageResult<>();

        try {
            customer.setUpdateBy(subject);
            customer.setUpdateTime(new Date());
            
            int affectCount = customerService.update(customer);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(customer);
                msgResult.setMessage("数据库更新成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),customer.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Customer customer = customerService.get(id);
            customer.setDelFlag(true);
            customer.setUpdateBy(subject);
            customer.setUpdateTime(new Date());

            int affectCount = customerService.update(customer);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Customer customer = customerService.get(id);
                customer.setDelFlag(true);
                customer.setUpdateBy(subject);
                customer.setUpdateTime(new Date());

                affectCount += customerService.update(customer);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String company, String name, String phone,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
		if (StringUtils.isNotEmpty(company)) {
            searchParams.put("company","%" + company + "%");
        }
        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }
        if (StringUtils.isNotEmpty(phone)) {
            searchParams.put("phone","%" + phone + "%");
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        Page<Customer> page = customerService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="select列表")
    @RequestMapping(value = "findList",method = RequestMethod.POST)
    public MessageResult findList(@RequestAttribute String subject){
        MessageResult msgResult = new MessageResult<>();

        List<Customer> list = customerService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }
}
