package com.jpsoft.printing.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.base.entity.Flaw;
import com.jpsoft.printing.modules.base.service.FlawService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/flaw")
@Api(description = "flaw")
public class FlawController {
    private Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
    private SysLogService sysLogService;
    @Autowired
    private FlawService flawService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Flaw> create(){
        MessageResult<Flaw> msgResult = new MessageResult<>();

        Flaw flaw = new Flaw();

        msgResult.setData(flaw);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Flaw> add(@RequestBody Flaw flaw,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Flaw> msgResult = new MessageResult<>();

        try {
            flaw.setId(UUID.randomUUID().toString());
            flaw.setDelFlag(false);
            flaw.setCreateBy(subject);
            flaw.setCreateTime(new Date());
            
            int affectCount = flawService.insert(flaw);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(flaw);
				msgResult.setMessage("数据库添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),flaw.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Flaw> edit(@PathVariable("id") String id){
        MessageResult<Flaw> msgResult = new MessageResult<>();

        try {
            Flaw flaw = flawService.get(id);

            if (flaw != null) {
                msgResult.setResult(true);
                msgResult.setData(flaw);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Flaw> update(@RequestBody Flaw flaw,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Flaw> msgResult = new MessageResult<>();

        try {
            flaw.setUpdateBy(subject);
            flaw.setUpdateTime(new Date());
            
            int affectCount = flawService.update(flaw);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(flaw);
				msgResult.setMessage("数据库更新成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),flaw.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Flaw flaw = flawService.get(id);
            flaw.setDelFlag(true);
            flaw.setUpdateBy(subject);
            flaw.setUpdateTime(new Date());

            int affectCount = flawService.update(flaw);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Flaw flaw = flawService.get(id);
                flaw.setDelFlag(true);
                flaw.setUpdateBy(subject);
                flaw.setUpdateTime(new Date());

                affectCount += flawService.update(flaw);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String name,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
		if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        Page<Flaw> page = flawService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
