package com.jpsoft.printing.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.base.entity.Staff;
import com.jpsoft.printing.modules.base.service.StaffService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/staff")
@Api(description = "staff")
public class StaffController {
    private Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
    private SysLogService sysLogService;
    @Autowired
    private StaffService staffService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Staff> create(){
        MessageResult<Staff> msgResult = new MessageResult<>();

        Staff staff = new Staff();

        msgResult.setData(staff);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Staff> add(@RequestBody Staff staff,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Staff> msgResult = new MessageResult<>();

        try {
            staff.setId(UUID.randomUUID().toString());
            staff.setDelFlag(false);
            staff.setCreateBy(subject);
            staff.setCreateTime(new Date());
            
            int affectCount = staffService.insert(staff);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(staff);
				msgResult.setMessage("数据库添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),staff.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Staff> edit(@PathVariable("id") String id){
        MessageResult<Staff> msgResult = new MessageResult<>();

        try {
            Staff staff = staffService.get(id);

            if (staff != null) {
                msgResult.setResult(true);
                msgResult.setData(staff);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Staff> update(@RequestBody Staff staff,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Staff> msgResult = new MessageResult<>();

        try {
            staff.setUpdateBy(subject);
            staff.setUpdateTime(new Date());
            
            int affectCount = staffService.update(staff);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(staff);
				msgResult.setMessage("数据库更新成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),staff.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Staff staff = staffService.get(id);
            staff.setDelFlag(true);
            staff.setUpdateBy(subject);
            staff.setUpdateTime(new Date());

            int affectCount = staffService.update(staff);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Staff staff = staffService.get(id);
                staff.setDelFlag(true);
                staff.setUpdateBy(subject);
                staff.setUpdateTime(new Date());

                affectCount += staffService.update(staff);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String serialNumber, String name,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
		if (StringUtils.isNotEmpty(serialNumber)) {
            searchParams.put("serialNumber","%" + serialNumber + "%");
        }
        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("serial_number","asc"));

        Page<Staff> page = staffService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
