package com.jpsoft.printing.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.printing.modules.base.entity.Work;
import com.jpsoft.printing.modules.base.service.WorkService;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.base.entity.Stock;
import com.jpsoft.printing.modules.base.service.StockService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/stock")
@Api(description = "stock")
public class StockController {
    private Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
    private SysLogService sysLogService;
    @Autowired
    private StockService stockService;
    @Autowired
    private WorkService workService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Stock> create(){
        MessageResult<Stock> msgResult = new MessageResult<>();

        Stock stock = new Stock();

        msgResult.setData(stock);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Stock> add(@RequestBody Stock stock,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Stock> msgResult = new MessageResult<>();

        try {
            stock.setId(UUID.randomUUID().toString());
            stock.setDelFlag(false);
            stock.setCreateBy(subject);
            stock.setCreateTime(new Date());
            
            int affectCount = stockService.insert(stock);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(stock);
				msgResult.setMessage("数据库添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),stock.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Stock> edit(@PathVariable("id") String id){
        MessageResult<Stock> msgResult = new MessageResult<>();

        try {
            Stock stock = stockService.get(id);

            if (stock != null) {
                msgResult.setResult(true);
                msgResult.setData(stock);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Stock> update(@RequestBody Stock stock,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Stock> msgResult = new MessageResult<>();

        try {
            stock.setUpdateBy(subject);
            stock.setUpdateTime(new Date());
            
            int affectCount = stockService.update(stock);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(stock);
				msgResult.setMessage("数据库更新成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),stock.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Stock stock = stockService.get(id);
            stock.setDelFlag(true);
            stock.setUpdateBy(subject);
            stock.setUpdateTime(new Date());

            int affectCount = stockService.update(stock);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Stock stock = stockService.get(id);
                stock.setDelFlag(true);
                stock.setUpdateBy(subject);
                stock.setUpdateTime(new Date());

                affectCount += stockService.update(stock);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String workId, String flaw, Boolean status,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
		if (StringUtils.isNotEmpty(workId)) {
            searchParams.put("workId",workId);
        }
        if (StringUtils.isNotEmpty(flaw)) {
            searchParams.put("flaw","%" + flaw + "%");
        }
        if (status != null) {
            searchParams.put("status",status);
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        Page<Stock> page = stockService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);
        for(Stock stock : page.getResult()){
            Work work = workService.get(stock.getWorkId());
            stock.setWorkName(work.getName());
            stock.setNumber(work.getNumber());
            stock.setColour(work.getColour());
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="批量出库")
    @PostMapping("batchOut")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchOut(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Stock stock = stockService.get(id);
                stock.setStatus(2);
                stock.setUpdateBy(subject);
                stock.setUpdateTime(new Date());

                affectCount += stockService.update(stock);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("出库成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("出库失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="撤回")
    @PostMapping("back/{id}")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> back(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Stock stock = stockService.get(id);
            stock.setStatus(1);
            stock.setUpdateBy(subject);
            stock.setUpdateTime(new Date());

            int affectCount = stockService.update(stock);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("撤回成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("撤回失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
