package com.jpsoft.printing.modules.base.controller;

import cn.hutool.core.convert.Convert;
import com.github.pagehelper.Page;
import com.jpsoft.printing.modules.base.entity.Account;
import com.jpsoft.printing.modules.base.entity.Customer;
import com.jpsoft.printing.modules.base.entity.Stock;
import com.jpsoft.printing.modules.base.service.AccountService;
import com.jpsoft.printing.modules.base.service.CustomerService;
import com.jpsoft.printing.modules.base.service.StockService;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.base.entity.Work;
import com.jpsoft.printing.modules.base.service.WorkService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/work")
@Api(description = "work")
public class WorkController {
    private Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
    private SysLogService sysLogService;
    @Autowired
    private CustomerService customerService;
    @Autowired
    private WorkService workService;
    @Autowired
    private AccountService accountService;
    @Autowired
    private StockService stockService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Work> create(){
        MessageResult<Work> msgResult = new MessageResult<>();

        Work work = new Work();
        work.setTagOperator("+");
        work.setTagUnit("米");
        work.setProcessDate(new Date());
        work.setIsInteger(false);

        msgResult.setData(work);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Work> add(@RequestBody Work work,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Work> msgResult = new MessageResult<>();

        try {
            work.setId(UUID.randomUUID().toString());
            work.setStatus(false);
            work.setDelFlag(false);
            work.setCreateBy(subject);
            work.setCreateTime(new Date());
            
            int affectCount = workService.insert(work);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(work);
				msgResult.setMessage("数据库添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),work.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Work> edit(@PathVariable("id") String id){
        MessageResult<Work> msgResult = new MessageResult<>();

        try {
            Work work = workService.get(id);

            if (work != null) {
                msgResult.setResult(true);
                msgResult.setData(work);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Work> update(@RequestBody Work work,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Work> msgResult = new MessageResult<>();

        try {
            work.setUpdateBy(subject);
            work.setUpdateTime(new Date());
            
            int affectCount = workService.update(work);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(work);
				msgResult.setMessage("数据库更新成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),work.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Work work = workService.get(id);
            work.setDelFlag(true);
            work.setUpdateBy(subject);
            work.setUpdateTime(new Date());

            int affectCount = workService.update(work);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
	@Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
		Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Work work = workService.get(id);
                work.setDelFlag(true);
                work.setUpdateBy(subject);
                work.setUpdateTime(new Date());

                affectCount += workService.update(work);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
				msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
			
			Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String customerId, String name, String number,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();
		if (StringUtils.isNotEmpty(customerId)) {
            searchParams.put("customerId",customerId);
        }
        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }
        if (StringUtils.isNotEmpty(number)) {
            searchParams.put("number","%" + number + "%");
        }

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        Page<Work> page = workService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);
        for(Work work : page.getResult()){
            Customer customer = customerService.get(work.getCustomerId());
            if(StringUtils.isNotEmpty(customer.getCompany())){
                work.setCustomerName(customer.getCompany());
            }

            Account account = accountService.getByWorkId(work.getId());
            if(account != null){
                work.setHaveBills(true);
            }
            else{
                work.setHaveBills(false);
            }

            BigDecimal totalLength = BigDecimal.ZERO;
            List<Stock> stockList = stockService.findList(work.getId());
            for(Stock stock : stockList) {
                if(work.getTagUnit().equals("码")){
                    if(work.getIsInteger()){
                        stock.setTagLength(stock.getTagLength().divide(new BigDecimal(0.914400), 0, BigDecimal.ROUND_HALF_UP));
                    }
                    else{
                        stock.setTagLength(stock.getTagLength().divide(new BigDecimal(0.914400), 1, BigDecimal.ROUND_HALF_UP));
                    }
                }
                else{
                    if(work.getIsInteger()){
                        stock.setTagLength(stock.getTagLength().setScale(0,BigDecimal.ROUND_HALF_UP));
                    }
                    else{
                        stock.setTagLength(stock.getTagLength().setScale(1,BigDecimal.ROUND_HALF_UP));
                    }
                }

                totalLength = totalLength.add(stock.getTagLength());
            }
            work.setTotalStock(stockList.size());
            work.setTotalLength(totalLength);

            if(work.getTagUnit() == null){
                work.setTagOperator(work.getTagOperator() + work.getTagFactor());
            }
            else{
                if(work.getTagUnit().equals("码")){
                    work.setTagOperator(work.getTagOperator() + work.getTagFactorSave() + "y");
                }
                else{
                    work.setTagOperator(work.getTagOperator() + work.getTagFactorSave() + "m");
                }
            }
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="select列表")
    @RequestMapping(value = "findList",method = RequestMethod.POST)
    public MessageResult findList(@RequestAttribute String subject){
        MessageResult msgResult = new MessageResult<>();

        List<Work> list = workService.list();

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }

    @ApiOperation(value="细码单数据")
    private Map dataXMD(String workId){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        Work work = workService.get(workId);
        Customer customer = customerService.get(work.getCustomerId());

        BigDecimal totalLength = new BigDecimal(0);
        Map<String, Object> map = new HashMap<>();
        List<Map> mapList = new ArrayList<>();
        List<Stock> stockList = stockService.findList(workId);
        for(int i=0; i<stockList.size(); i++){
            Stock stock = stockList.get(i);

            if(work.getTagUnit().equals("码")){
                if(work.getIsInteger()){
                    stock.setTagLength(stock.getTagLength().divide(new BigDecimal(0.914400),0,BigDecimal.ROUND_HALF_UP));
                }
                else{
                    stock.setTagLength(stock.getTagLength().divide(new BigDecimal(0.914400),1,BigDecimal.ROUND_HALF_UP));
                }
            }
            else{
                if(work.getIsInteger()){
                    stock.setTagLength(stock.getTagLength().setScale(0,BigDecimal.ROUND_HALF_UP));
                }
                else{
                    stock.setTagLength(stock.getTagLength().setScale(1,BigDecimal.ROUND_HALF_UP));
                }
            }

            totalLength = totalLength.add(stock.getTagLength());

            Map<String, Object> accMap = new HashMap<>();
            accMap.put("sort", stock.getStockNumber());
            accMap.put("length", stock.getTagLength());
            mapList.add(accMap);
        }

        map.put("customer", customer.getCompany());
        map.put("number", work.getNumber());
        map.put("name", work.getName());
        map.put("width", work.getWidth());
        map.put("colour", work.getColour());
        map.put("unit", work.getTagUnit());
        map.put("list", mapList);
        map.put("totalCount", stockList.size());
        map.put("totalLength", totalLength);

        return map;
    }

    @ApiOperation(value="细码单")
    @RequestMapping(value = "tableXMD",method = RequestMethod.POST)
    public MessageResult tableXMD(String workId){
        MessageResult msgResult = new MessageResult<>();

        Map map = dataXMD(workId);

        msgResult.setResult(true);
        msgResult.setData(map);

        return msgResult;
    }

    @ApiOperation(value="细码单报表")
    @RequestMapping(value = "tableXMDXls",method = RequestMethod.POST)
    public MessageResult tableXMDXls(String workId) throws Exception {
        MessageResult msgResult = new MessageResult<>();

        Map map = dataXMD(workId);

        String xlsPath = "static/xmd.xls";
        ClassPathResource resource = new ClassPathResource(xlsPath);
        Workbook wb = WorkbookFactory.create(resource.getInputStream());

        //Excel
        Sheet sheetTitle = wb.getSheetAt(0);
        //数据
        List<Map> list = (List<Map>)map.get("list");
        Row rowTitle = sheetTitle.getRow(2);
        rowTitle.getCell(0).setCellValue("客户：" + map.get("customer").toString());
        rowTitle.getCell(4).setCellValue("订单号：" + map.get("number").toString());
        rowTitle.getCell(9).setCellValue("品名：" + map.get("name").toString());
        rowTitle.getCell(14).setCellValue("幅宽：" + map.get("width").toString());
        rowTitle.getCell(18).setCellValue("单位：" + map.get("unit").toString());
        int rowStart = 5;
        //分页
        int pageNum = (int)Math.ceil(list.size() / 200);
        for(int i = 0; i < pageNum; i++) {
            Sheet newSheet = wb.cloneSheet(0);
            newSheet.setSelected(false);
            wb.setSheetName(wb.getSheetIndex(newSheet), "sheet" + (i + 2));
        }

        BigDecimal colLenTotal = new BigDecimal(0);
        BigDecimal pageLenTotal = new BigDecimal(0);
        Integer pageSize = 0;
        for(int i = 0; i < list.size(); i++) {
            Map item = list.get(i);
            int columnNum = (int)Math.floor((i % 200) / 20);
            int rowNum = i % 20;

            Sheet sheet = wb.getSheetAt((int)Math.floor(i / 200));
            sheet.getRow(rowStart + rowNum).getCell(columnNum * 2).setCellValue(item.get("sort").toString());
            sheet.getRow(rowStart + rowNum).getCell(columnNum * 2 + 1).setCellValue(item.get("length").toString());

            colLenTotal = colLenTotal.add(Convert.toBigDecimal(item.get("length")));
            pageLenTotal = pageLenTotal.add(Convert.toBigDecimal(item.get("length")));
            pageSize++;

            if((i + 1) % 20 == 0 || i + 1 == list.size()) {
                sheet.getRow(3).getCell(columnNum * 2).setCellValue(map.get("colour").toString());
                sheet.getRow(25).getCell(columnNum * 2).setCellValue(colLenTotal.toString());
                colLenTotal = new BigDecimal(0);
            }

            if((i + 1) % 200 == 0 || i + 1 == list.size()) {
                sheet.getRow(27).getCell(0).setCellValue(pageSize.toString());
                sheet.getRow(27).getCell(3).setCellValue(pageLenTotal.toString());
                pageLenTotal = new BigDecimal(0);
                pageSize = 0;
            }
        }

        //保存到服务器，返回文件名
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        wb.write(output);
        String fileName = "细码单（" + sdf.format(new Date()) + "）.xls";
//        String name = "D:\\" + fileName;//调试
        String name = "D:\\JpSoft\\Tomcat 8.5\\webapps\\printing-portal\\xls\\" + fileName;
        File file = new File(name);
        FileOutputStream file1 = new FileOutputStream(file);
        wb.write(file1);
        file1.close();
        wb.close();
        output.close();

        msgResult.setResult(true);
        msgResult.setMessage("报表生成完成。");
        msgResult.setData("/printing-portal/xls/" + fileName);

        return msgResult;
    }
}
