package com.jpsoft.printing.modules.common.controller;

import com.jpsoft.printing.config.OSSConfig;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.common.utils.OSSUtil;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RestController
public class fileUploadController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private SysLogService sysLogService;
    @Autowired
    private OSSConfig ossConfig;

    @PostMapping("uploadPicture")
    @ApiOperation(value="照片上传")
    @Transactional(rollbackFor = Exception.class)
    @ApiImplicitParams({
            @ApiImplicitParam(name="subFolder",value = "路径",required = true,paramType = "form"),
            @ApiImplicitParam(name = "photoFile",value = "员工照片", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<Map> uploadPicture(String subFolder, MultipartFile photoFile, @RequestAttribute String subject, HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

            String fileName = photoFile.getOriginalFilename();
            String rawFileUrl = OSSUtil.upload(ossConfig,"/" + subFolder,sdf.format(new Date()) + fileName.substring(fileName.indexOf(".")),photoFile.getInputStream());

            Map<String,Object> dataMap = new HashMap<>();
            dataMap.put("fileName", fileName);
            dataMap.put("fileUrl", rawFileUrl);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
            messageResult.setCode(200);

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),null,end-begin,null,messageResult.getMessage());
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }
}
