package com.jpsoft.printing.modules.sys.controller;

import com.github.pagehelper.Page;
import com.jpsoft.printing.config.OSSConfig;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.utils.OSSUtil;
import com.jpsoft.printing.modules.common.utils.POIUtils;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.sys.entity.DataDictionary;
import com.jpsoft.printing.modules.sys.entity.User;
import com.jpsoft.printing.modules.sys.service.DataDictionaryService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import com.jpsoft.printing.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.*;

@RestController
@RequestMapping("/sys/dataDictionary")
@Api(description = "数据字典")
public class DataDictionaryController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private SysLogService sysLogService;
    @Autowired
    private DataDictionaryService dataDictionaryService;
    @Autowired
    private UserService userService;
    @Autowired
    private OSSConfig ossConfig;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<DataDictionary> create(){
        MessageResult<DataDictionary> msgResult = new MessageResult<>();

        DataDictionary dataDictionary = new DataDictionary();

        msgResult.setData(dataDictionary);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<DataDictionary> add(@RequestBody DataDictionary dataDictionary, @RequestAttribute String subject, HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<DataDictionary> msgResult = new MessageResult<>();

        try {
            dataDictionary.setId(UUID.randomUUID().toString());
            dataDictionary.setDelFlag(false);
            dataDictionary.setCreateBy(subject);
            dataDictionary.setCreateDate(new Date());

            int affectCount = dataDictionaryService.insert(dataDictionary);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(dataDictionary);
                msgResult.setMessage("添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("添加失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),dataDictionary.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage("添加失败");
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<DataDictionary> edit(@PathVariable("id") String id){
        MessageResult<DataDictionary> msgResult = new MessageResult<>();

        try {
            DataDictionary dataDictionary = dataDictionaryService.get(id);

            if (dataDictionary != null) {
                msgResult.setResult(true);
                msgResult.setData(dataDictionary);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<DataDictionary> update(@RequestBody DataDictionary dataDictionary,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<DataDictionary> msgResult = new MessageResult<>();

        try {
            dataDictionary.setUpdateBy(subject);
            dataDictionary.setUpdateDate(new Date());

            int affectCount = dataDictionaryService.update(dataDictionary);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(dataDictionary);
                msgResult.setMessage("更新成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("更新失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),dataDictionary.toString(),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage("更新失败");
        }

        return msgResult;
    }

    @ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            DataDictionary dataDictionary = dataDictionaryService.get(id);
            dataDictionary.setDelFlag(true);
            dataDictionary.setUpdateBy(subject);
            dataDictionary.setUpdateDate(new Date());

            int affectCount = dataDictionaryService.update(dataDictionary);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                DataDictionary dataDictionary = dataDictionaryService.get(id);
                dataDictionary.setDelFlag(true);
                dataDictionary.setUpdateBy(subject);
                dataDictionary.setUpdateDate(new Date());

                affectCount += dataDictionaryService.update(dataDictionary);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            String name,
            String parentId,
            @RequestParam(name="pageIndex",defaultValue = "1") int pageIndex,
            @RequestParam(name="pageSize",defaultValue = "10") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("sort_no","asc"));

        if (StringUtils.isNotEmpty(parentId)) {
            searchParams.put("parentId",parentId);
        }

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id",id);
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name","%" + name + "%");
        }

        Page<DataDictionary> page = dataDictionaryService.pageSearch(searchParams,pageIndex, pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value = "查询")
    @RequestMapping(value = "query", method = RequestMethod.POST)
    public MessageResult<List> query(
            String keywords,
            String excludeId,
            @RequestParam(value = "dataType", defaultValue = "null") String dataType,
            @RequestParam(value = "limit", defaultValue = "20") int limit,
            @RequestAttribute String subject) {
        MessageResult<List> msgResult = new MessageResult<>();

        Map<String, Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.sort_no", "asc"));

        if (StringUtils.isNotEmpty(keywords)) {
            searchParams.put("name", "%" + keywords + "%");
        }

        if (StringUtils.isNotEmpty(excludeId)) {
            searchParams.put("excludeId", excludeId);
        }

        if (StringUtils.isNotEmpty(dataType)) {
            //1查目录-2查值
            searchParams.put("dataType", dataType);
        }


        Page<DataDictionary> page = dataDictionaryService.pageSearch(searchParams,0, limit,sortList);
        List<DataDictionary> dataDictionaryList = page.getResult();
        msgResult.setResult(true);
        msgResult.setData(dataDictionaryList);

        return msgResult;
    }


    @ApiOperation(value = "查询字典")
    @RequestMapping(value = "queryChildren", method = RequestMethod.POST)
    public MessageResult<List> queryChildren(
            @RequestParam(value = "parentId", defaultValue = "") String parentId,
            @RequestAttribute String subject) {
        MessageResult<List> msgResult = new MessageResult<>();

        List<Map<String, Object>> dataDictionaryList = dataDictionaryService.queryChildren(parentId);
        msgResult.setResult(true);
        msgResult.setData(dataDictionaryList);
        return msgResult;
    }

    @PostMapping("findByCatalogName")
    @ApiOperation(value = "根据目录名称查询数据列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "catalogName",value = "数据目录名称", required = true, paramType = "form",dataType = "String"),
    })
    public MessageResult<List> findByCatalogName(String catalogName){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            //todo
            List<DataDictionary> list = dataDictionaryService.findByCatalogName(catalogName);
            messageResult.setData(list);

            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @ApiOperation(value="导入用户")
    @PostMapping("importXls")
    @Transactional(rollbackFor = Exception.class)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "uploadFile",value = "上传文件", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<String> importXls(MultipartFile uploadFile, @RequestAttribute String subject,HttpServletRequest request){
        Long begin = System.currentTimeMillis();
        User user = userService.get(subject);

        MessageResult<String> msgResult = new MessageResult<>();

        try {
            POIUtils poiUtils = new POIUtils(uploadFile.getInputStream());
            int sheetIndex = 0;
            Sheet sheet1 = poiUtils.getSheetAt(sheetIndex);

            int affectCount = 0;
            int failCount = 0;
            int validateColIndex = 4;

            for(int rowIndex=1; rowIndex<=sheet1.getLastRowNum(); rowIndex++){
                try {
                    String sortNo = (String)poiUtils.getCellValue(sheetIndex,rowIndex,0).toString().replace(" ","");
                    String value = (String)poiUtils.getCellValue(sheetIndex,rowIndex,1).toString().replace(" ","");
//                    String position = (String)poiUtils.getCellValue(sheetIndex,rowIndex,2).toString().replace(" ","");
                    String name = (String)poiUtils.getCellValue(sheetIndex,rowIndex,3).toString().replace(" ","");

                    DataDictionary dataDictionary = new DataDictionary();
                    dataDictionary.setId(UUID.randomUUID().toString());
                    dataDictionary.setName(name);
                    dataDictionary.setValue(value);
                    dataDictionary.setSortNo(Integer.parseInt(sortNo));
                    dataDictionary.setParentId("1775997f-b69d-41ea-83ac-83f345be5d97");
                    dataDictionary.setDataType("2");
                    dataDictionary.setDelFlag(false);
                    dataDictionary.setCreateBy(subject);
                    dataDictionary.setCreateDate(new Date());
                    dataDictionaryService.insert(dataDictionary);

                    affectCount++;
                }
                catch(Exception innerEx){
                    sheet1.getRow(rowIndex).createCell(validateColIndex).setCellValue("错误!");
                    failCount++;
                    logger.error(innerEx.getMessage(),innerEx);
                }
            }

            if (failCount>0){
                //有导入失败的记录
                msgResult.setResult(false);
                msgResult.setMessage("数据成功导入" + affectCount + "条,有" + failCount + "条数据未导入成功，错误原因请查看报表。");

                //todo 只保留错误数据的sheet
                Workbook wb = poiUtils.exportErrorXls(sheetIndex,validateColIndex,1 + affectCount + failCount);

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                wb.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                String downloadUrl = OSSUtil.upload(ossConfig,"import","error.xls",input);

                //todo 返回导入失败报表下载链接
                msgResult.setData(downloadUrl);
                msgResult.setMessage(downloadUrl);
            }
            else{
                msgResult.setResult(true );
                msgResult.setMessage("数据成功导入" + affectCount + "条");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),null,end-begin,null,msgResult.getMessage());
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
