package com.jpsoft.printing.modules.sys.controller;

import com.github.pagehelper.Page;
import com.jpsoft.printing.modules.common.dto.PropsDTO;
import com.jpsoft.printing.modules.common.utils.PojoUtils;
import com.jpsoft.printing.modules.common.dto.Sort;
import com.jpsoft.printing.modules.common.dto.MessageResult;
import com.jpsoft.printing.modules.sys.entity.Menu;
import com.jpsoft.printing.modules.sys.service.MenuService;
import com.jpsoft.printing.modules.sys.service.SysLogService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@RestController
@RequestMapping("/sys/menu")
public class MenuController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private SysLogService sysLogService;
    @Autowired
    private MenuService menuService;

    @ApiOperation(value = "创建空记录")
    @GetMapping("create")
    public MessageResult<Menu> create() {
        MessageResult<Menu> msgResult = new MessageResult<>();

        Menu menu = new Menu();

        msgResult.setData(menu);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value = "添加信息")
    @PostMapping("add")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Menu> add(@RequestBody Menu menu, @RequestAttribute String subject, HttpServletRequest request) {
        Long begin = System.currentTimeMillis();
        MessageResult<Menu> msgResult = new MessageResult<>();

        try {
            menu.setId(UUID.randomUUID().toString());
            menu.setDelFlag(false);
            menu.setCreateBy(subject);
            menu.setCreateTime(new Date());

            int affectCount = menuService.insert(menu);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(menu);
                msgResult.setMessage("数据库添加成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),menu.toString(),end-begin,null,msgResult.getMessage());
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Menu> edit(@PathVariable("id") String id) {
        MessageResult<Menu> msgResult = new MessageResult<>();

        try {
            Menu menu = menuService.get(id);

            if (menu != null) {
                msgResult.setResult(true);
                msgResult.setData(menu);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "更新用户")
    @PostMapping("update")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Menu> update(@RequestBody Menu menu, @RequestAttribute String subject,HttpServletRequest request) {
        Long begin = System.currentTimeMillis();
        MessageResult<Menu> msgResult = new MessageResult<>();

        try {
            menu.setUpdateBy(subject);
            menu.setUpdateTime(new Date());

            int affectCount = menuService.update(menu);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(menu);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),menu.toString(),end-begin,null,msgResult.getMessage());
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "删除")
    @PostMapping("delete/{id}")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> delete(@PathVariable("id") String id, @RequestAttribute String subject,HttpServletRequest request) {
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Menu menu = menuService.get(id);
            menu.setDelFlag(true);
            menu.setUpdateBy(subject);
            menu.setUpdateTime(new Date());

            int affectCount = menuService.update(menu);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),id,end-begin,null,msgResult.getMessage());
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value = "批量删除")
    @PostMapping("batchDelete")
    @Transactional(rollbackFor = Exception.class)
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList, @RequestAttribute String subject,HttpServletRequest request) {
        Long begin = System.currentTimeMillis();
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Menu menu = menuService.get(id);
                menu.setDelFlag(true);
                menu.setUpdateBy(subject);
                menu.setUpdateTime(new Date());

                affectCount += menuService.update(menu);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
                msgResult.setMessage("删除成功");
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }

            Long end = System.currentTimeMillis();
            sysLogService.addLog(subject,request.getRemoteAddr(),request.getServletPath(),String.join(",",idList),end-begin,null,msgResult.getMessage());
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value = "查询")
    @RequestMapping(value = "query", method = RequestMethod.POST)
    public MessageResult<List> query(
            String keywords,
            String excludeId,
            @RequestAttribute String subject) {
        MessageResult<List> msgResult = new MessageResult<>();

        Map<String, Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
//        sortList.add(new Sort("a.parent_id", "asc"));
        sortList.add(new Sort("a.sort_no", "asc"));

        if (StringUtils.isNotEmpty(keywords)) {
            searchParams.put("menuName", "%" + keywords + "%");
        }

        if (StringUtils.isNotEmpty(excludeId)) {
            searchParams.put("excludeId", excludeId);
        }

        List<PropsDTO> list = new ArrayList<>();
        List<Menu> menuList = menuService.query(searchParams, 1000, sortList);
        for(Menu menu : menuList) {
            PropsDTO dto = new PropsDTO();
            dto.setId(menu.getId());
            dto.setName(menu.getMenuName());
            dto.setMenuName(menu.getMenuName());
            dto.setParentId(menu.getParentId());
            dto.setParentName(menu.getParentName());
            list.add(dto);
        }

        msgResult.setResult(true);
        msgResult.setData(list);

        return msgResult;
    }

    @ApiOperation(value = "列表")
    @RequestMapping(value = "pageList", method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String menuName,
            String menuType,
            String parentId,
            @RequestParam(name="pageIndex",defaultValue = "1") int pageIndex,
            @RequestParam(name="pageSize",defaultValue = "10") int pageSize,
            @RequestAttribute String subject) {

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String, Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("sort_no", "asc"));

        if (StringUtils.isNotEmpty(menuName)) {
            searchParams.put("menuName", "%" + menuName + "%");
        }

        if (StringUtils.isNotEmpty(menuType)) {
            searchParams.put("menuType", menuType);
        }

        if (StringUtils.isNotEmpty(parentId)) {
            searchParams.put("parentId", parentId);
        }

        Page<Menu> page = menuService.pageSearch(searchParams, pageIndex, pageSize, sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value = "获取授权菜单")
    @GetMapping("tree")
    public MessageResult<List> tree(@RequestAttribute("subject") String subject) {
        MessageResult<List> msgResult = new MessageResult<>();

        List<Menu> list = menuService.findAllocMenu(subject,null);

        findChildrenLoop(list,subject,0,5);

        msgResult.setData(list);
        msgResult.setResult(true);

        return msgResult;
    }

    private void findChildrenLoop(List<Menu> list, String subject,int deep,int max) {
        if (deep<max) {
            for (Menu menu : list) {
                List<Menu> children = menuService.findAllocMenu(subject,menu.getId());

                menu.setChildren(children);

                findChildrenLoop(children,subject,deep+1,max);
            }
        }
    }
}
